package com.fanruan.hihidata.datasource.server;

import com.fanruan.hihidata.datasource.server.http.HttpKits;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import org.jetbrains.annotations.NotNull;
import org.json.JSONObject;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

/**
 * @author richie
 * Created by richie on 2022/9/16
 */
public class Authentication {

    private final String accessKeyId;
    private final String accessKeySecret;

    private final LoadingCache<String, String> cache = CacheBuilder.newBuilder()
            .expireAfterWrite(30, TimeUnit.MINUTES)
            .build(new CacheLoader<String, String>() {

                @Override
                public String load(@NotNull String key) throws Exception {
                    return fetchAccessToken();
                }
            });

    public Authentication(String accessKeyId, String accessKeySecret) {
        this.accessKeyId = accessKeyId;
        this.accessKeySecret = accessKeySecret;
    }

    private String fetchAccessToken() throws Exception {
        String res = HttpKits.get(String.format("%s/api/v1/corp/oauth/token?accessKeyId=%s&accessKeySecret=%s", ProjectConstants.BASE_URI, accessKeyId, accessKeySecret));
        JSONObject jsonObject = new JSONObject(res);
        if (jsonObject.optBoolean("success", false)) {
            return jsonObject.getString("data");
        }
        return null;
    }

    public String getToken() {
        try {
            return cache.get("ticket");
        } catch (ExecutionException e) {
            return null;
        }
    }
}
